from django.db import models


class GenericManager(models.Manager):
    """
    Filter query set by given selectors
    """
    def __init__(self, **kw):
        super(GenericManager, self).__init__()
        self.selectors = kw

    def get_query_set(self):
        return super(GenericManager, self).get_query_set().filter(
            **self.selectors
        )



import markdown as md

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe


register = template.Library()


@register.filter
@stringfilter
def markdown(string):
    return mark_safe(md.markdown(string))



import random
from base64 import urlsafe_b64encode, urlsafe_b64decode

from django.conf import settings


ALPHABET = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'


class URLEncoder(object):
    def __init__(self, secret, alphabet=ALPHABET, noise_length=12):
        self.secret = secret
        self.alphabet = alphabet
        self.noise_length = noise_length

    @staticmethod
    def xor(str1, str2):
        return ''.join(chr(ord(a) ^ ord(b)) for a, b in zip(str1, str2))

    def noise(self, length):
        return ''.join([random.choice(self.alphabet) for i in range(length)])

    def encode(self, string):
        string = self.noise(self.noise_length) + string
        return urlsafe_b64encode(
            self.xor(string, self.secret)
        ).replace('=', '')

    def encode_id(self, id):
        return self.encode(str(id))

    def decode(self, string):
        if len(string) % 4:
            add = (4 - len(string) % 4) * '='
        else:
            add = ''
        return self.xor(
            urlsafe_b64decode(str(string + add)), self.secret
        )[self.noise_length:]

    def decode_id(self, string):
        return int(self.decode(string))


# Set URL_ENCODER_KEY in settings
# Make sure it is different to the SECRET_KEY setting
encoder = URLEncoder(settings.URL_ENCODER_KEY)



import json
from functools import wraps

from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext


def as_json(func):
    def decorator(request, *ar, **kw):
        output = func(request, *ar, **kw)

        if not isinstance(output, dict):
            return output

        return HttpResponse(json.dumps(output), 'application/json')
    return decorator


def as_html(template_path):
    """
    Decorator with the same functionality as render_to_response has, but uses
    decorator syntax.
    """

    def decorator(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            output = func(request, *args, **kwargs)

            if not isinstance(output, dict):
                return output

            return render_to_response(
                template_path,
                output,
                context_instance=RequestContext(request)
            )
        return wrapper
    return decorator



from django.http import HttpResponse


class HttpMethodNotAllowed(HttpResponse):
    status_code = 405


class BaseView(object):
    allowed_methods = ['get']

    def __new__(cls, *ar, **kw):
        return super(BaseView, cls).__new__(cls)(*ar, **kw)

    def __call__(self, request, *ar, **kw):
        method = request.method.lower()
        if method not in self.allowed_methods:
            return HttpMethodNotAllowed('Not allowed: %s' % method.upper())
        method_call = getattr(self, method)
        return method_call(request, *ar, **kw)

    def get(self, request, *ar, **kw):
        raise NotImplementedError

    def post(self, request, *ar, **kw):
        raise NotImplementedError

    def put(self, request, *ar, **kw):
        raise NotImplementedError

    def delete(self, request, *ar, **kw):
        raise NotImplementedError



from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage


def page_objects(qs, by, page=None):
    paginator = Paginator(qs, by)
    page = page or 1
    try:
        page = paginator.page(page)
    except PageNotAnInteger:
        page = paginator.page(1)
    except EmptyPage:
        page = paginator.page(paginator.num_pages)

    return page



import os
from setuptools import setup, find_packages

from tiamat import __doc__, __version__, __author__, __email__


desc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()


setup(
    name='django-tiamat',
    version=__version__,
    author=__author__,
    author_email=__email__,
    url='https://bitbucket.org/wearefarm/django-tiamat',
    description=__doc__,
    long_description=desc,
    license='BSD',
    packages=find_packages(),
    install_requires=['Django >= 1.4'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django'
    ]
)



from django import template


register = template.Library()


@register.simple_tag
def render_form(form):
    """
    Render a form quickly using a generic template

    Usage:
        <form>{% csrf_token %}
        {% render_form form %}
        <input type="submit>
        </form>
    """
    t = template.loader.get_template('tiamat/_form.html')
    return t.render(template.Context({'form': form}))
